package at.xaxa.ledger.ui.category.edit

import android.util.Log
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.setValue
import androidx.lifecycle.SavedStateHandle
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import at.xaxa.ledger.data.EntryRepository
import at.xaxa.ledger.data.db.Category.CategoryEntity
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

data class CategoryUIState(val category: CategoryEntity = CategoryEntity(0,"",0))



class EditCategoryViewModel(private val savedStateHandle: SavedStateHandle, private val entryRepository: EntryRepository) : ViewModel() {


    private val categoryId: Int = checkNotNull(savedStateHandle["categoryId"])


    var categoryUi by mutableStateOf(CategoryUIState())
        private set
    var deletePossible by mutableStateOf(false)
    var redirect by mutableStateOf(false)



    init {
        viewModelScope.launch {
            val category = withContext(Dispatchers.IO) {
                entryRepository.findCategoryById(categoryId)
            }
            categoryUi = CategoryUIState(category)

            redirect = false;
        }
    }

    fun updateCategory(category: CategoryEntity) {
        categoryUi = categoryUi.copy(category = category)
    }

    fun deleteCategory() {
        viewModelScope.launch {
            entryRepository.deleteCategory(categoryUi.category)
        }
    }

    fun saveCategory() {
        viewModelScope.launch {
            entryRepository.updateCategory(categoryUi.category)
        }
    }

    fun findEntryByCategoryId() {
        viewModelScope.launch {
            val fetchedEntries = withContext(Dispatchers.IO) {
              entryRepository.findEntryByCategoryId(categoryId)
            }

            if(fetchedEntries>0){
                deletePossible = true
            }else{
                deleteCategory()
                redirect= true;
            }
        }
        //print("ASASDADS$categoryId")
    }
}
